#import "Logger.h"
#import "RuntimeHelpers.h"

#pragma mark - Logging

#define LogError(desc, ...) DoNotCallThisDirectlyLogCommon(Logger.sharedInstance, LogLevelError, (desc), ##__VA_ARGS__)
#define LogWarn(desc, ...) DoNotCallThisDirectlyLogCommon(Logger.sharedInstance, LogLevelWarn, (desc), ##__VA_ARGS__)
#define LogInfo(desc, ...) DoNotCallThisDirectlyLogCommon(Logger.sharedInstance, LogLevelInfo, (desc), ##__VA_ARGS__)
#define LogVerbose(desc, ...) DoNotCallThisDirectlyLogCommon(Logger.sharedInstance, LogLevelVerbose, (desc), ##__VA_ARGS__)
#define LogPrivate(desc, ...) DoNotCallThisDirectlyLogCommon(Logger.sharedInstance, LogLevelPrivate, (desc), ##__VA_ARGS__)

#define DoNotCallThisDirectlyLogCommon(logger, lvl, desc, ...) \
    if ([logger shouldLogForLevel:(lvl)]) { \
        [logger log:(lvl) function:[[NSString alloc] initWithUTF8String:__FUNCTION__] file:[[NSString alloc] initWithUTF8String:__FILE__] line:__LINE__ format:(desc), ##__VA_ARGS__]; \
    } \

#pragma mark - Helpers

#define LogFunctionEntry() \
    [Logger.sharedInstance.logEntryIntoFunction:[NSString alloc] initWithUTF8String:__FUNCTION__] file:[[NSString alloc] initWithUTF8String:__FILE__] line:__LINE__]; \

#pragma mark - Assertions

#define LogAssert(condition, desc, ...) DoNotCallThisDirectlyLogAssertCommon((condition), Logger.sharedInstance, NO, (desc), ##__VA_ARGS__)
#define LogAssertFailed(desc, ...) DoNotCallThisDirectlyLogAssertCommon(NO, Logger.sharedInstance, NO, (desc), ##__VA_ARGS__)
#define LogAssertWithStack(condition, desc, ...) DoNotCallThisDirectlyLogAssertCommon((condition), Logger.sharedInstance, YES, (desc), ##__VA_ARGS__)
#define LogAssertFailedWithStack(desc, ...) DoNotCallThisDirectlyLogAssertCommon(NO, Logger.sharedInstance, YES, (desc), ##__VA_ARGS__)

#define DoNotCallThisDirectlyLogAssertCommon(condition, logger, shouldIncludeStack, desc, ...) \
    if (!(condition)) \
        [logger logFailedAssertion:(shouldIncludeStack) function:[[NSString alloc] initWithUTF8String:__FUNCTION__] file:[[NSString alloc] initWithUTF8String:__FILE__] line:__LINE__ format:(desc), ##__VA_ARGS__]; \
        if (logger.shouldReallyAssert) { \
            NSAssert((condition), (desc), ##__VA_ARGS__); \
        } else if (IsDebuggerAttached()) \
            BreakIntoDebugger(); \
        } \
    } \
