#import <Foundation/Foundation.h>

@protocol BuildInfoProtocol;

typedef NS_ENUM(NSInteger, LogLevel) {
    LogLevelError,
    LogLevelWarn,
    LogLevelInfo,
    LogLevelVerbose,
    LogLevelPrivate // This is a special log level which is used to conditionally log PII.
};

NS_ASSUME_NONNULL_BEGIN

@protocol LoggerImpl

- (BOOL)shouldLogForLevel:(LogLevel)level;
- (void)logMessage:(NSString *)message level:(LogLevel)level function:(NSString *)function file:(NSString *)file line:(NSUInteger)line;
- (void)clear;

@end

@interface Logger : NSObject <LoggerImpl>

@property (class) Logger *sharedInstance NS_SWIFT_NAME(shared);

@property (nonatomic, readonly) BOOL isTestBuild;
@property (nonatomic, readonly) BOOL shouldReallyAssert;

- (instancetype)init NS_UNAVAILABLE;
- (instancetype)initWithBuildInfo:(id<BuildInfoProtocol>)buildInfo impl:(id<LoggerImpl>)impl NS_DESIGNATED_INITIALIZER;
- (void)logFailedAssertionWithMessage:(NSString *)message shouldIncludeStack:(BOOL)shouldIncludeStack function:(NSString *)function file:(NSString *)file line:(NSUInteger)line;
- (void)logEntryIntoFunction:(NSString *)function file:(NSString *)file line:(NSUInteger)line NS_SWIFT_NAME(logEntry(into:file:line:));

@end

@interface Logger (ObjectiveC)

- (void)log:(LogLevel)level function:(NSString *)function file:(NSString *)file line:(NSUInteger)line format:(NSString *)format, ... NS_FORMAT_FUNCTION(5, 6);
- (void)logFailedAssertion:(BOOL)shouldIncludeStack function:(NSString *)function file:(NSString *)file line:(NSUInteger)line format:(NSString *)format, ... NS_FORMAT_FUNCTION(5, 6);

@end

NS_ASSUME_NONNULL_END
